# $Id: TTOBTclInterp.tcl,v 1.1 1998/06/04 22:25:24 pb Exp $
# Copyright 1995 TipTop Software

#############################################################################
class TTOBCCTclInterpProcs [TTOBCCategory] {
} {
method -(NSString*)stringValue {
  return [@ "commands"]
}
  
method -(void)populateArray:(NSMutableArray*)l withSubcellsForCell:(TTOBCell*)c {
  set i [$c object]

  if ![tt $i] return

  if [catch {$i evalString: {::info procs}} r] return
  set p [lsort $r]

  set c2 [TTOBCLabel object]
  $c2 setStringValue: [@ "[llength $p] procs"]
  $l addObject: $c2

  foreach x $p {
    set c2 [TTOBCTclInterpProc newCopyOf: $c]
    $c2 setProcName: [@ $x]
    $l addObject: $c2
  }

  if [catch {$i evalString: {::info commands}} r] return
  set cmd [lsort $r]

  set label [TTOBCLabel object]
  $l addObject: $label

  set count 0
  foreach x $cmd {
    if {[lsearch -exact $p $x]!=-1} continue
    if [string match {[+-]\[* *\]*} $x] continue
    set c2 [TTOBCTclInterpProc newCopyOf: $c]
    $c2 setProcName: [@ $x]
    $l addObject: $c2
    incr count
  }

  $label setStringValue: "$count builtins"
}
}

#############################################################################
class TTOBCTclInterpProc TTOBCell {
} {
method -(void)setProcName:(NSString*)n {
  $self setStringValue: $n
}

method -(NSString*)procName {
  return [$self stringValue]
}

method -(BOOL)loadCellAtRow:(int)row inColumn:(int)col {
  $self setLeaf: yes
  $self setLoaded: yes
  return yes
}

method -(NSString*)populateArray:(NSMutableArray*)l \
                   withInspectorClassNamesForCategory:(TTOBCCategory*)t {
  $l addObject: [@ "TTOBTclInterpProcInspector"]
  return [@ "TTOBTclInterpProcInspector"]
}

method -(BOOL)canBeRootCell { return no }

method -(NSImage*)getCellImage { return [TTObjBrowser getImageNamed: [@ TclProc]] }
method -(NSString*)getCellName { return [$self procName] }
method -(NSString*)dragData { return [$self procName] }
}

#############################################################################
class TTOBCCTclInterpGlobals TTOBCCategory {
} {
method -(NSString*)stringValue {
  return [@ "globals"]
}

method -(void)populateArray:(NSMutableArray*)l withSubcellsForCell:(TTOBCell*)c {
  set i [$c object]

  if ![tt $i] return
  if [catch {$i evalString: {::info globals}} r] return
  set g [lsort $r]

  # Put label cell:
  set c2 [TTOBCLabel object]
  $c2 setStringValue: [@ "[llength $g] globals"]
  $l addObject: $c2

  # Put other cells:
  set brow [$c browser]
  foreach x $g {
    if {[catch {$i getVariable: [@ $x]} val]==0 \
        && [catch {objtcl::lookup $val} obj]==0} {
      set c2 [$brow newCellForObject: $obj]
      $c2 setStringValue: [@ $x]
    } else {
      if {[catch {$i evalString: "::array size $x"} arr] || $arr==0} {
        set c2 [TTOBCTclInterpGlobal object]
      } else {
        set c2 [TTOBCTclInterpGlobalArray object]
      }
      $c2 setBrowser: $brow
      $c2 setObject: $i
      $c2 setGlobalName: [@ $x]
    }
    $l addObject: $c2
  }
}
}

#############################################################################
class TTOBCTclInterpGlobal TTOBCell {
  NSString *globalName;
  BOOL isArrayElement;
  NSString *element;
} {
method -(void)dealloc {
  set globalName [nil]
  set element [nil]
  super dealloc
}

method -(void)setGlobalName:(NSString*)n {
  set globalName $n
}

method -(NSString*)globalName {
  return $globalName
}

method -(void)setElement:(NSString*)e {
  set isArrayElement yes
  set element $e
}

method -(NSString*)element {
  return $element
}

method -(BOOL)isArrayElement {
  return $isArrayElement
}

method -(void)_preload {
  if $isArrayElement {
    $self setStringValue: $element
  } else {
    $self setStringValue: $globalName
  }
}

method -(BOOL)loadCellAtRow:(int)row inColumn:(int)col {
  $self setLeaf: yes
  $self setLoaded: yes
  return yes
}

method -(NSString*)populateArray:(NSMutableArray*)l \
                   withInspectorClassNamesForCategory:(TTOBCCategory*)t {
  $l addObject: [@ "TTOBTclInterpGlobalInspector"]
  return [nil]
}

method -(BOOL)canBeRootCell { return no }

method -(NSImage*)getCellImage {
  return [TTObjBrowser getImageNamed: [@ TclGlobal]]
}
method -(NSString*)getCellName { return [$self globalName] }
method -(NSString*)dragData { return [$self globalName] }
}

#############################################################################
class TTOBCTclInterpGlobalArray TTOBCTclInterpGlobal {
} {
method -(NSString*)dragData {
  return [$self getCellName]
}

method -(void)_preload {
  $self setStringValue: "[* $globalName]()"
}

method -(BOOL)loadCellAtRow:(int)row inColumn:(int)col {
  $self setLeaf: no
  $self setLoaded: yes
  return yes
}

method -(NSString*)populateArray:(NSMutableArray*)l \
                   withInspectorClassNamesForCategory:(TTOBCCategory*)t {
  $l addObject: [@ "TTOBTclInterpGlobalArrayInspector"]
  return [nil]
}

method -(NSString*)populateArrayWithSubcellCategories:(NSMutableArray*)l {
  $l addObject: [TTOBTclCCArrayContents object]
  return [nil]
}

method -(NSImage*)getCellImage {
  return [TTObjBrowser getImageNamed: [@ TclArray]]
}

method -(NSString*)getCellName {
  return "array $globalName"
}
}

#############################################################################
class TTOBTclCCArrayContents TTOBCCategory {
} {
method -(NSString*)stringValue {
  return [@ "Tcl array contents"]
}

method -(void)populateArray:(NSMutableArray*)l withSubcellsForCell:(TTOBCell*)c {
  set interp [$c object]
  if ![tt $interp] return
  set gnx [$c globalName]
  set gn [* $gnx]
  if {![catch {$interp evalString: "::array names $gn"} names] \
      && [llength $names]>0} {
    set names [lsort $names]

    set brow [$c browser]
    set j 0

    foreach name $names {
      set val [$interp getVariable: "${gn}($name)"]
      set c2 [TTOBCTclInterpGlobal object]
      $c2 setBrowser: $brow
      $c2 setObject: $interp
      $c2 setGlobalName: $gnx
      $c2 setElement: [@ $name]
      $l addObject: $c2
    }
  }
}

}

#EOF