# $Id: TTOBTclInterpInspector.tcl,v 1.1 1998/06/04 22:25:24 pb Exp $
# Copyright 1995 TipTop Software

#############################################################################
class TTOBTclInterpProcInspector TTOBInspector {
  id procNameField;
  id procText;
} {
method +(NSString*)inspectorName { return [@ "Command"] }

method -(BOOL)wantsButtons { return yes }

method - (void)dealloc {
  $procText setDelegate: [nil]
  super dealloc
}

method - init {
  $self initAndLoadNib
  $procText setDelegate: $self
  $procText setFont: [NSFont userFixedPitchFontOfSize: 10]
  return $self
}

method -(void)revert:sender {
  set pn [$info procName]
  set i [$info object]

  if [catch {$i evalString: "info body $pn"} r] {
    $procText setString: [@ "<Compiled-C>"]
    $procText setBackgroundColor: [NSColor lightGrayColor]
    $procText setEditable: no
    $procNameField setStringValue: "builtin $pn"
  } else {
    $procText setBackgroundColor: [NSColor whiteColor]
    $procText setEditable: yes
    set body $r

    set args [$i evalString: "::info args $pn"]
    set aa {}
    foreach a $args {
      if [catch {$i evalString: "list \[::info default $pn $a __tmp__\] \[set __tmp__\]"} def] {
        lappend aa $a
        continue
      }
      if {[lindex $def 0]==0} {
        lappend aa $a
      } else {
        lappend aa [list $a [lindex $def 1]]
      }
    }
    $i evalString: {catch {unset __tmp__}}

    set pr [list proc $pn $aa $body]
    $procText setString: [@ $pr]
    $procNameField setStringValue: [@ "proc $pn"]
  }
  super revert: $sender
}

method -(void)ok: sender {
  if ![$procText isEditable] {
    NSBeep
    self revert: $sender
    return
  }
  set pr [list [* [$procText string]]]
  set i [$info object]

  if [catch {$i evalString: "eval $pr"} r] {
    NSRunAlertPanel [nil] "$r" [nil] [nil] [nil]
  }
  super ok: $sender
  $self revert: $sender
}
}

#############################################################################
class TTOBTclInterpGlobalInspector TTOBInspector {
  id nameField;
  id valueText;
} {
  method +(NSString*)inspectorName { return [@ "Global"] }

  method - (void)dealloc {
    $valueText setDelegate: [nil]
    super dealloc
  }

  method -init {
    $self initAndLoadNib
    $valueText setDelegate: $self
    $valueText setFont: [NSFont userFixedPitchFontOfSize: 10]
    return $self
  }

  method -(void)revert: sender {
    set i [$info object]
    set gn [$info globalName]

    if [$info isArrayElement] {
      set el [* [$info element]]
      set val [$i getVariable: "${gn}($el)"]
      $nameField setStringValue: [@ ${gn}($el)]
    } else {
      set val [$i getVariable: $gn]
      $nameField setStringValue: $gn
    }
    $valueText setString: [@ $val]

    super revert: $sender
  }

  method -(void)ok: sender {
    set i [$info object]
    set gn [$info globalName]

    if [$info isArrayElement] {
      set el [* [$info element]]
      $i setVariable: ${gn}($el) toValue: [$valueText string]
    } else {
      $i setVariable: $gn toValue: [$valueText string]
    }
    super revert: $sender
  }

  method -(BOOL)wantsButtons { return yes }
}

#############################################################################
class TTOBTclInterpGlobalArrayInspector TTOBInspector {
  id nameField;
  id valueForm;
} {
method +(NSString*)inspectorName { return [@ "Global array"] }
method -(BOOL)wantsButtons { return yes }

method - (void)dealloc {
  $valueForm setDelegate: [nil]
  super dealloc
}

method -init {
  $self initAndLoadNib
  $valueForm setTextFont: [NSFont userFixedPitchFontOfSize: 10]
  $valueForm setScrollable: yes
  $valueForm setTarget: $self;
  $valueForm setAction: touch:
  $valueForm setDelegate: $self;
  return $self
}

method -(void)revert: sender {
  set i [$info object]
  set gn [$info globalName]
  $nameField setStringValue: $gn

  $window disableFlushWindow

  # Remove (old) entries
  set cnt [$valueForm numberOfRows]
  for {set j [expr $cnt-1]} {$j>=0} {incr j -1} {
    $valueForm removeEntryAtIndex: $j
  }

  if ![catch {$i evalString: "array names $gn"} names] {
    set names [lsort $names]

    set j 0
    foreach name $names {
      $valueForm addEntry: [@ "$name:"]
      set val [$i getVariable: ${gn}($name)]
      [$valueForm cellAtIndex: $j] setStringValue: [@ $val]
      incr j
    }
  }
  $valueForm sizeToCells
  $window enableFlushWindow
  $window flushWindowIfNeeded

  super revert: $sender
}

method -(void)ok: sender {
  set i [$info object]
  set gn [$info globalName]
  set cnt [$valueForm numberOfRows]
  for {set j 0} {$j<$cnt} {incr j} {
    set el [[$valueForm cellAtIndex: $j] title]
    set el [string range $el 0 [expr [string length $el]-2]]
    $i setVariable: ${gn}($el) \
      toValue: [[$valueForm cellAtIndex: $j] stringValue]
  }
  super revert: $sender
}
}

#############################################################################
class TTOBTclInterpInspector TTOBInspector {
} {
method +(NSString*)inspectorName { return [@ "Interp"] }

method -init {
  return [$self initAndLoadNib]
}

method -(void)doShowInteractor: sender {
  set i [$info object]
  set r [[TTWInteractor alloc] init]
  $r setReleasedWhenClosed: yes
  $r setInterp: $i
  $r runModal: no
}

method -(BOOL)wantsButtons { return no }
}

# EOF
