// $Id: TTInterp.h,v 5.8 1998/07/02 05:25:13 pb Exp $
/* Copyright (c) 1994-1998 TipTop Software, Inc.
   All Rights Reserved

   Author: Pedja Bogdanovich
*/
#ifndef _ObjCore_TTInterp_h
#define _ObjCore_TTInterp_h 1

#import <ObjCore/ObjCoreDefines.h>

#import <Foundation/NSObject.h>
#import <Foundation/NSDictionary.h>
#import <ObjCore/TTInterpProtocol.h>


@interface TTInterp : TTObject
{
@protected
  unsigned _iflags;
  id _interactor;
}

+ (void)registerLanguageClass:(Class)c;
+ (void)unregisterLanguageClass:(Class)c;

- (BOOL)isDead;

+ (NSString*)interpClassNameForLanguageName:(NSString*)lang;
+ (Class)getClassForLanguageName:(NSString *)name;

+ (NSDictionary*)allLoadedLanguages;
+ (BOOL)loadLanguageName:(NSString*)l;
+ (BOOL)loadAppKit;
+ (BOOL)loadBrowser;

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aDecoder;
@end

@interface TTInterp(SubclassesShouldOverride)
// Return default interp; create and initialize if needed.
+ (TTInterp*)getDefaultInterp;

// Array of extension names
+ (NSArray*)sourceFileTypes;
- (NSArray*)sourceFileTypes;

// Language name
+ (NSString*)languageName;
- (NSString *)languageName;

// Indicate whether mutliple interpreter instances can be created.
+ (BOOL)isMultiInterp;
@end

@interface TTInterp(Value)
// Value class.
+ (Class)interpValueClass;
- (id)objectForValue:(id <TTInterpValueProtocol>)value flags:(unsigned)f;
- (id <TTInterpValueProtocol>)valueForObject:(id)obj flags:(unsigned)f;
@end

@class NSImage;
@interface TTInterp(AppKitStuff)
// Optional: used for window title in interactor.
- (NSString*)shortDescription;

// Prefix for IB variable connections.
+(NSString*)keyNamePrefix;

// IB image.
+(NSImage*)imageForViewer;
@end

@interface TTInterp(Convenience)
// Subclasses respond the following convenience methods:
- (void)evalFile:(NSString*)filename;
- (id <TTInterpValueProtocol>)evalString:(NSString*)code;
- (void)setVariable:(NSString*)key toValue:(id <TTInterpValueProtocol>)value;
- (id <TTInterpValueProtocol>)getVariable:(NSString*)key;
- (void)takeValue:(id)o forKey:(NSString*)k;
- (id)valueForKey:(NSString*)k;
+ (NSRange)expressionRange:(NSString *)code forRange:(NSRange)range;
@end

@interface TTInterp(LibPath)
- (NSArray*)libPath;
- (void)addToLibPath:(NSString*)dir;
- (void)setLibPath:(NSArray*)dirarr;
- (void)addAllBundleLibraryPaths;
@end

#ifdef  __cplusplus
extern "C" {
#endif

OBJCORE_EXTERN NSString *TTLanguageObjC;
OBJCORE_EXTERN NSString *TTLanguageTcl;
OBJCORE_EXTERN NSString *TTLanguagePerl;
OBJCORE_EXTERN NSString *TTLanguageJava;
OBJCORE_EXTERN NSString *TTLanguageJavaScript;
OBJCORE_EXTERN NSString *TTLanguageVisualBasic;
OBJCORE_EXTERN NSString *TTLanguagePython;
OBJCORE_EXTERN NSString *TTLanguageWebScript;
OBJCORE_EXTERN NSString *TTLanguageAppleScript;

OBJCORE_EXTERN NSString *TTScopeGlobal;
OBJCORE_EXTERN NSString *TTScopeLocal;

OBJCORE_EXTERN NSString *TTDebugNone;
OBJCORE_EXTERN NSString *TTDebugInteractor;

#ifdef  __cplusplus
}
#endif

#endif
// EOF
